﻿using System;
using System.Collections;
using System.IO;
using System.Net.Http;
using System.Text;
using UnityEditor;
using UnityEditor.AssetImporters;
using UnityEngine;
using Zepeto;
using ZEPETO.Asset;
using Object = UnityEngine.Object;

public class ZepetoStudioLoader : MonoBehaviour
{
    public const string VERSION = "1.0.0";

    public const string METADATA_URL =
        "https://api-studio.zepeto.me/api/settings/v1/asset_bundle_meta";

    [Serializable]
    public class ZepetoStudioMetadata
    {
        private static ZepetoStudioMetadata _studioMetadata;
        private static bool isInitialized;
        public static ZepetoStudioMetadata Metadata
        {
            get
            {
                if (_studioMetadata == null && isInitialized == false)
                {
                    isInitialized = true;
                    using (var client = new HttpClient())
                    {
                        var response = client.GetAsync(METADATA_URL).Result;
                        if (response.IsSuccessStatusCode)
                        {
                            var responseContent = response.Content;
                            string responseString = responseContent.ReadAsStringAsync().Result;
                            _studioMetadata = JsonUtility.FromJson<ZepetoStudioMetadata>(responseString);
                        }
                    }    
                }

                if (_studioMetadata == null)
                {
                    EditorUtility.DisplayDialog("ERROR", "Invalid studio metadata. please try again", "ok");
                    return null; 
                }

                if (VERSION != _studioMetadata.latestVersion)
                {
                    if (EditorUtility.DisplayDialog("New version", "A new version of zepeto studio has been released.",
                        "Download New version"))
                    {
                        Application.OpenURL(string.IsNullOrEmpty(_studioMetadata.downloadLink)
                            ? "https://studio.zepeto.me"
                            : _studioMetadata.downloadLink);
                    }
                    return null;
                }

                return _studioMetadata;
            }
        }
        [Serializable]
        public class AssetBundleMeta
        {
            public int polygonCount;
            public int textureCount;
            public int textureSize;
            public int vertexCount;
        }
        
        public string latestVersion;
        public string downloadLink;
        public AssetBundleMeta assetBundleMeta;
    }
    
    private ZepetoCharacterCustomLoader _loader;
    public TextAsset[] Deformations;

    private void Start()
    {
        _loader = GetComponent<ZepetoCharacterCustomLoader>();
        _loader.enabled = true;
    }

    private GUIStyle guiStyle = new GUIStyle(); //create a new variable

    public void OnGUI()
    {
        if(_loader == null) return;


        if (this._loader.Context != null && this._loader.Context.IsInitialized)
        {
            float guiScale = Screen.width / 512.0f;
            var oldMatrix = GUI.matrix;
            GUI.matrix = Matrix4x4.Scale(new Vector3(guiScale, guiScale, guiScale)) *
                         Matrix4x4.Translate(new Vector3(0, 0, 0));


            GUILayout.BeginArea(new Rect(512 - 100, 0, 100, 512));
            GUILayout.BeginVertical();
            guiStyle.fontSize = 10;
            guiStyle.normal.textColor = Color.black;
            GUILayout.Label("Deformations ", guiStyle);
            if (GUILayout.Button("DEFAULT"))
            {
                _loader.Context.Metadata.SetDeformations(new ZepetoMetadata.DeformationItem[0]);
            }   
            foreach (var deformation in Deformations)
            {
                if(deformation == null) continue;
                if (GUILayout.Button(deformation.name))
                {
                    var deformations = ZepetoMetadata.DeformationItem.Parse(deformation.text);
                    _loader.Context.Metadata.SetDeformations(deformations);

                }    
            }
        
            GUILayout.EndVertical();
            GUILayout.EndArea();

            GUI.matrix = oldMatrix;
        }
    }
}

public class Menu {
    /////////////////////////////////////////////
    /// CLOTH
    /////////////////////////////////////////////
    [MenuItem("Assets/Zepeto Studio/Convert to ZEPETO style", false)]
    public static void ConvertCloth()
    {
        ZepetoModelImporter.ConvertAsset<ZepetoClothConverter>();
    }

    /////////////////////////////////////////////
    /// EXPORT
    /////////////////////////////////////////////
    [MenuItem("Assets/Zepeto Studio/Export as .zepeto", false, 100)]
    public static void Archive()
    {
        if (Selection.activeObject == null)
            return;
        
        var path = AssetDatabase.GetAssetPath(Selection.activeObject);
        var isTexture = Selection.activeObject is Texture2D;
        if (isTexture) {
            ShowMakeupConvertWindow();
            return;
        } 
        
        var result = new StringBuilder();
        var isPrefab = string.IsNullOrEmpty(path) == false && Path.GetExtension(path).Equals(".prefab", StringComparison.OrdinalIgnoreCase);
        if (isPrefab) {
            var childRenderers = Selection.activeGameObject.GetComponentsInChildren<SkinnedMeshRenderer>();

            if (childRenderers == null || childRenderers.Length == 0) return;

            var statistics = ZepetoAssetBundleInfo.Create(Selection.activeGameObject);

            result.AppendLine("[ZEPETO STUDIO ARCHIVE RESULT]");

            var json = JsonUtility.ToJson(statistics, true);
            result.AppendLine("==============================");
            result.AppendLine(json);
            result.AppendLine("==============================");
        }
        var packResult = ZepetoAssetPackage.Pack(Selection.activeObject, isPrefab);
        result.Append(packResult);

        Debug.Log(result);
    }
    
    /////////////////////////////////////////////
    /// MAKEUP
    /////////////////////////////////////////////
    public static void ShowMakeupConvertWindow() {
        var path = AssetDatabase.GetAssetPath(Selection.activeObject);
        var outputPath = ZepetoMakeupImporter.GetOutputAssetPath(path);
        if (File.Exists(outputPath)) {
            Debug.LogError($"please try again after deleting the file: {outputPath}");
            return;
        }
        var window = EditorWindow.GetWindow<MakeupConvertWindow>(true, "Convert Makeup", true);
        window.maxSize = new Vector2(300, 250);
        window.minSize = new Vector2(300, 250);
        window.ShowModalUtility();
    }
    
    private class MakeupConvertWindow : EditorWindow {
        private string[] options = new[] {
            ZepetoTextureCustomizer.BlendModes.None.ToString(),
            ZepetoTextureCustomizer.BlendModes.Multiply.ToString(),
            ZepetoTextureCustomizer.BlendModes.Screen.ToString(),
            ZepetoTextureCustomizer.BlendModes.ColorDodge.ToString(),
            ZepetoTextureCustomizer.BlendModes.LinearDodge.ToString(),
            ZepetoTextureCustomizer.BlendModes.Overlay.ToString(),
        };
        private int selected;
        private void OnGUI() {
            EditorGUILayout.BeginVertical();
            GUILayout.Space(20);
            var mode = (ZepetoTextureCustomizer.BlendModes)Enum.Parse(typeof(ZepetoTextureCustomizer.BlendModes), options[selected]);
            GUILayout.Label($"Selected BlendMode: {mode.ToString()}");
            selected = GUILayout.SelectionGrid(selected, options, 1);
            GUILayout.FlexibleSpace();
            EditorGUILayout.BeginHorizontal();
            if (GUILayout.Button("Cancel", GUILayout.Height(30))) {
                Close();
            }
            if (GUILayout.Button("Convert", GUILayout.Height(30))) {
                try {
                    Create();
                } catch {
                    //
                } finally {
                    Close();
                }
            }
            EditorGUILayout.EndHorizontal();
            EditorGUILayout.EndVertical();
        }

        private void Create() {
            var path = AssetDatabase.GetAssetPath(Selection.activeObject);
            var convertedPath = ZepetoMakeupImporter.GetOutputAssetPath(path);

            var texture = Selection.activeObject as Texture2D;
            var masked = MakeupConverter.CullingMakeupTexture(texture);
            if (texture == null || texture.width != texture.height) {
                Debug.LogError("choose a texture with a 1:1 ratio");
                return;
            }
            
            var assetName = Path.GetFileNameWithoutExtension(path);
            var maskedPath = $"{Path.GetDirectoryName(path)}/{assetName}_{GUID.Generate()}.png";
            File.WriteAllBytes(maskedPath, masked.EncodeToPNG());
            Object.DestroyImmediate(masked);
            
            AssetDatabase.Refresh();

            var outputPath = ZepetoMakeupImporter.GetOutputAssetPath(maskedPath);
            var mode = (ZepetoTextureCustomizer.BlendModes)Enum.Parse(typeof(ZepetoTextureCustomizer.BlendModes), options[selected]);
            Selection.activeObject = AssetDatabase.LoadAssetAtPath<Object>(maskedPath);
            ZepetoMakeupImporter.ConvertMakeup(mode);
            File.Delete(maskedPath);
            File.Delete($"{maskedPath}.meta");

            File.Delete(convertedPath);
            File.Move(outputPath, convertedPath);
            File.Move($"{outputPath}.meta", $"{convertedPath}.meta");
            
            AssetDatabase.Refresh();
            
            var asset = AssetDatabase.LoadAssetAtPath<Object>(convertedPath);
            var packResult = ZepetoAssetPackage.Pack(asset, false);
            File.Delete(convertedPath);
            File.Delete($"{convertedPath}.meta");
            AssetDatabase.Refresh();
            Debug.Log(packResult);
        }
    }
}

[ScriptedImporter(1, "zepeto")]
public class ZepetoStudioImporter : ScriptedImporter
{
    public override void OnImportAsset(AssetImportContext ctx)
    {
        var texture = Resources.Load<Texture2D>("zepeto-studio-icon");
        if(texture == null) return;
        ctx.AddObjectToAsset("Texture", texture, texture);
    }
}

public class ZepetoImporter : AssetPostprocessor {
    private void OnPreprocessAsset() {
        if (Directory.Exists(assetPath) == false) return;
        AssetDatabase.ImportAsset(assetPath, ImportAssetOptions.ImportRecursive);
    }
    
    private void OnPreprocessModel() {
        if (assetImporter is ModelImporter modelImporter) {
            modelImporter.avatarSetup = ModelImporterAvatarSetup.CreateFromThisModel;
            modelImporter.meshOptimizationFlags = MeshOptimizationFlags.PolygonOrder;
            modelImporter.isReadable = true;
        }
    }

    private void OnPreprocessTexture() {
        if (assetImporter is TextureImporter textureImporter) {
            textureImporter.isReadable = true;
        }
    }
}
